import logging
import subprocess
import sys

class _AbortCommand(Exception):
    def __init__(self, code, msg):
        self.code = code
        self.msg = msg

class BaseCommand(object):
    name = None
    help = None

    def __init__(self, subparsers):
        self.parser = subparsers.add_parser(self.name, help=self.help)
        self.parser.set_defaults(func=self.execute_wrapper)
        self.init_logging()
        self.create_arguments()

    def init_logging(self):
        mname = self.__class__.__module__
        self.log = logging.getLogger('%s:%s' % (mname, self.name))

    def create_arguments(self):
        pass

    def abort(self, msg=None, code=2):
        raise _AbortCommand(code, msg)

    def execute_wrapper(self, parser, args):
        try:
            retval = self.execute(args)
            if retval is None:
                retval = 0
            parser.exit(retval)
        except _AbortCommand, e:
            msg = e.msg
            if msg:
                msg = msg.rstrip() + '\n'
            parser.exit(e.code, msg)

    def execute(self, args):
        raise NotImplimentedError('The Command object "%s" did not have an "execute" method')

class SystemProcessMixin(object):

    def sysproc(self, *args, **kwargs):
        self.log.vdebug('cmd args: %s', args)
        for arg in args:
            if not isinstance(arg, basestring):
                raise TypeError('arg %s is %s, but needs to be a string' % (arg, type(arg)))
        proc = subprocess.Popen(
                args,
                stdin=subprocess.PIPE,
                stderr=subprocess.PIPE,
                stdout=subprocess.PIPE,
                **kwargs
            )
        stdout, stderr = proc.communicate('123\n')
        self.log.debug('cmd return code: %s', proc.returncode)
        self.log.debug('cmd stdout: %s', stdout)
        self.log.debug('cmd stderr: %s', stderr)
        return proc.returncode, stdout, stderr

    def checkcall(self, *args, **kwargs):
        """
            Same as sysproc, but will check the returncode and if non-zero
            will print stdout, stderror, and raise an exception that
            will prevent any further commands from being run.
        """
        ok_codes = kwargs.pop('ok_codes', [0])
        rc, so, se = self.sysproc(*args, **kwargs)
        if rc not in ok_codes:
            if so:
                print so
            if se:
                print >> sys.stderr, se
            self.abort()
        return rc, so, se



from decimal import Decimal
import os
from os import path
from blazeutils.config import QuickSettings
from blazeutils.datastructures import OrderedDict
from blazeutils.helpers import tolist, csvtolist
from blazech.exceptions import ConfigError
from blazech.lib.ConfigParser import SafeConfigParser
import blazech.locations as locations

class DefaultConfig(QuickSettings):

    def __init__(self):
        QuickSettings.__init__(self)

        #######################################################################
        # LOGGING
        #######################################################################
        # the full path to the file that should contain log files
        self.logging.mainlog.path = None
        # see the logging section of the docs for a description of these levels
        self.logging.mainlog.verbosity = 'minimal'
        # the full path to the file that should contain log files
        self.logging.errorlog.path = None
        # maximum log file size is 50MB
        self.logging.rotation.max_bytes = 1024*1024*50
        # number of rotated log files to keep around
        self.logging.rotation.backup_count = 4
        # default verbosity for console
        self.console.verbosity = 'minimal'
        # include errors on console output
        self.console.inc_errors = False

def parse_config_files(include_system_configs, fpaths):
    """
        fpaths: an interable of file system paths to configuration files that
        need to be processed.  We start with the default config and add the
        configuration file settings in order.
    """
    config_paths = []
    if include_system_configs:
        if path.isfile(locations.system_config_file):
            config_paths.append(locations.system_config_file)
        if path.isfile(locations.user_config_file):
            config_paths.append(locations.user_config_file)
    config_paths.extend(tolist(fpaths))

    df = DefaultConfig()
    if config_paths:
        cp = SafeConfigParser(dict_type=OrderedDict)
        cp.read(config_paths)
        df.update(_config_obj_to_qs(cp))
    df.lock()
    return df

def _config_obj_to_qs(config):
    """
        converts a config parser object to a quick settings object
    """
    qs = QuickSettings()

    # do the defaults
    for n,v in config.defaults().iteritems():
        v = _auto_convert_str_to_numeric_type(v)
        qs.set_dotted(n, v)

    # sectin settings get attached to their sections, we assume section naming
    # will be appropriate, i.e. [bulk], [jobs.mysql5], etc.
    all_sections = config.sections()
    for section in all_sections:
        for n,v in config.items(section):
            v = _auto_convert_str_to_numeric_type(v)
            if n == '_include_':
                sec_to_update = qs.get_dotted(section)
                for incsection in csvtolist(v):
                    if incsection not in all_sections:
                        raise ConfigError('included section %s not found' % incsection)
                    qs.get_dotted(section).update(qs.get_dotted(incsection))
            else:
                qs.set_dotted('%s.%s' % (section, n), v)
    return qs

def _auto_convert_str_to_numeric_type(string):
    try:
        return int(string)
    except ValueError, e:
        if 'invalid literal for int()' not in str(e):
            raise
    try:
        return Decimal(string)
    except Exception, e:
        if 'Invalid literal for Decimal' not in str(e):
            raise
    return string



import logging
import os
from os.path import join as pjoin
import shlex
import subprocess
import zipfile
from blazeutils.helpers import csvtolist, Timer
from blazeutils.datastructures import OrderedDict

class BaseAction(object):
    
    def __init__(self, batch, settings):
        self.batch = batch
        self.settings = settings
        self.stats = OrderedDict()
        self.timer = Timer()
        self._setup_logging()
    
    def _setup_logging(self):
        cname = self.__class__.__name__
        self.log = logging.getLogger('blazech.actions.%s' % cname)
    
    def execute(self):
        pass
    
    def elapsed_stat(self, name):
        self.stats['elapsed_' + name] = self.timer.elapsed(name)

class TestUseOnly(BaseAction):
    """ only use this for testing"""

    def execute(self):
        self.log.info('test output')
        
class RaiseException(BaseAction):
    """ only use this for testing"""
    
    def execute(self):
        assert False, 'this is meant for testing'

class SystemCommand(BaseAction):
    
    def runcmd(self, args, stdin=''):
        self.log.vdebug('cmd args: %s', args)
        for arg in args:
            if not isinstance(arg, basestring):
                raise TypeError('arg %s is %s, but needs to be a string' % (arg, type(arg)))
        proc = subprocess.Popen(
                args,
                stdin=subprocess.PIPE,
                stderr=subprocess.PIPE,
                stdout=subprocess.PIPE,
            )
        stdout, stderr = proc.communicate(stdin)
        self.log.debug('cmd return code: %s', proc.returncode)
        self.log.debug('cmd stdout: %s', stdout)
        self.log.debug('cmd stderr: %s', stderr)
        return proc.returncode, stdout, stderr
    
    def execute(self):
        cmd_str = self.settings.cmd
        args = shlex.split(cmd_str)
        self.log.debug('executing: %s', cmd_str)
        self.runcmd(args)      
        
class DatabaseBackupBase(SystemCommand):
    
    @property
    def dbnames(self):
        try:
            touse = csvtolist(self.settings.include)
            self.log.debug('using dbnames from settings.include: %s', touse)
            return touse
        except AttributeError, e:
            if 'no attribute \'include\'' not in str(e):
                raise
        all_db_names = self.fetch_all_dbnames()
        self.log.debug('fetched db names: %s', all_db_names)
        try:
            for dbname in csvtolist(self.settings.exclude):
                all_db_names.remove(dbname)
            self.log.debug('excluded dbnames: %s', self.settings.exclude)
        except AttributeError, e:
            if 'no attribute \'exclude\'' not in str(e):
                raise
        return all_db_names
            
    def sql_file_path(self, dbname):
        return pjoin(self.settings.target_dir, dbname + '.sql')
        
    def zip_file_path(self, dbname):
        return pjoin(self.settings.target_dir, dbname + '.zip')

    def perform_compression(self, dbname):
        if self.file_compression == 'zip':
            zipf = zipfile.ZipFile(self.zip_file_path(dbname), "w", zipfile.ZIP_DEFLATED )
            sql_file = self.sql_file_path(dbname)
            zipf.write(sql_file, os.path.basename(sql_file) )
            zipf.close()
            if self.after_compression == 'delete_originals':
                os.unlink(sql_file)
    
    def execute(self):
        self.dbport = getattr(self.settings, 'port', '3306')
        self.file_compression = getattr(self.settings, 'file_compression', 'zip')
        self.after_compression = getattr(self.settings, 'after_compression', 'delete_originals')
        
        self.timer.start('total')
        for dbname in self.dbnames:
            self.log.minimal('backing up db: %s', dbname)
            self.timer.start('backup_db_%s' % dbname)
            resultcode, stdout, stderr = self.perform_backup(dbname)
            self.elapsed_stat('backup_db_%s' % dbname)
            if resultcode == 0:
                self.timer.start('zip_db_%s' % dbname)
                self.perform_compression(dbname)
                self.elapsed_stat('zip_db_%s' % dbname)
            else:
                self.log.minimal('mysqldump command failed for db: %s', dbname)
        self.elapsed_stat('total')
        self.log.info('stats: %s', self.stats)

class MysqlBackup(DatabaseBackupBase):
    
    def fetch_all_dbnames(self):
        self.timer.start('elapsed_list_dbs')
        returncode, stdout, stderr = self.runcmd(self.list_db_args())
        self.elapsed_stat('elapsed_list_dbs')
        
        if returncode != 0:
            self.log.minimal('list databases command failed')
            return tuple()
        
        # the first row is the column header "Databases" so we strip that off
        # then every line is a DB name
        return stdout.split('\n')[1:]
    
    def common_args(self):
        return (
            '-h',
            self.settings.host,
            '-u',
            self.settings.user,
            '-p%s' %  self.settings.password,
            '-P',
            str(self.dbport),
        )
    
    def backup_args(self, dbname):
        return (pjoin(self.settings.bin_path, 'mysqldump'), ) + \
            self.common_args() + \
            (
            '-r',
            self.sql_file_path(dbname),
            dbname
        )
        
    def list_db_args(self):
        return (pjoin(self.settings.bin_path, 'mysql'), ) + \
            self.common_args() + \
            (
            '--batch',
            '-e',
            'show databases'
        )
    
    def perform_backup(self, dbname):
        return self.runcmd(self.backup_args(dbname))



from nose.tools import eq_, raises
from nose.plugins.skip import SkipTest
from blazech.config import DefaultConfig
from blazech.exceptions import ConfigError
import blazech.log
from blazeutils.testing import logging_handler, clear_test_handlers, ListIO
from helpers import run_batch, TestConfig, get_config_path, run_bch

#this code is from pysjobs, hasn't been converted as part of bch
raise SkipTest

@raises(ConfigError)
def test_value_error_with_bad_logging_config():
    settings = TestConfig()
    settings.logging.mainlog.verbosity = 'foobar'
    try:
        run_batch(settings)
    except ConfigError, e:
        if 'unrecognized value for logging.mainlog.verbosity: foobar' in str(e):
            raise

class TestBatchLogs(object):

    @classmethod
    def teardown_class(cls):
        clear_test_handlers()

    def test_succesful_job_at_batch_level(self):
        settings = TestConfig()
        settings.job.test._action_ = 'test_use_only'
        lh = logging_handler('blazech.Batch')
        result = run_batch(settings)
        assert 'batch started' in lh.current, lh.current
        assert 'test_use_only which maps to TestUseOnly' in lh.next, lh.current
        assert 'running job test' in lh.next, lh.current
        assert 'batch completed' in lh.next, lh.current

    def test_action_class_that_doesnt_exist(self):
        settings = TestConfig()
        settings.job.test._action_ = 'not_really_there'
        lh = logging_handler('blazech.Batch', 10)
        result = run_batch(settings)
        assert 'action class NotReallyThere can not be imported' in lh.next, lh.current

    def test_job_causes_exception(self):
        settings = TestConfig()
        settings.job.test._action_ = 'raise_exception'
        lh = logging_handler('blazech.Batch', 10)
        result = run_batch(settings)
        lh.next
        assert 'job test raised an exception' in lh.next, lh.current

    def test_no_jobs(self):
        settings = TestConfig()
        lh = logging_handler('blazech.Batch', 10)
        result = run_batch(settings)
        assert 'job test could not be run: no jobs have been configured' in lh.next, lh.current

    def test_no_job_settings(self):
        settings = TestConfig()
        settings.job.otherjob._action_ = 'raise_exception'
        lh = logging_handler('blazech.Batch', 10)
        result = run_batch(settings)
        assert 'job test could not be run: its settings were not found' in lh.next, lh.current

    def test_no_action_settings(self):
        settings = TestConfig()
        settings.job.test.foo = 'bar'
        lh = logging_handler('blazech.Batch', 10)
        result = run_batch(settings)
        assert 'job test could not be run: it did not have an _action_ setting' in lh.next, lh.current

class TestStdoutLogging(object):

    @classmethod
    def setup_class(cls):
        cls.cstream = ListIO()
        blazech.log.CONSOLE_STREAM = cls.cstream

    @classmethod
    def teardown_class(cls):
        blazech.log.CONSOLE_STREAM = None

    def tearDown(self):
        self.cstream.reset()

    def test_default_stdout_logging(self):
        settings = DefaultConfig()
        run_batch(settings)
        assert 'batch started' in self.cstream.current, self.cstream.current
        assert 'no jobs have been configured' in self.cstream.next, self.cstream.current
        assert 'batch completed' in self.cstream.next, self.cstream.current

    def test_console_logging_off(self):
        settings = TestConfig()
        run_batch(settings)
        assert len(self.cstream.contents) == 0

    def test_inc_errors_is_false(self):
        settings = DefaultConfig()
        settings.job.test._action_ = 'raise_exception'
        run_batch(settings)
        assert len(self.cstream.contents) == 4
        assert self.cstream.contents[2].strip().endswith('job test raised an exception')

    def test_inc_errors_is_true(self):
        settings = DefaultConfig()
        settings.console.inc_errors = True
        settings.job.test._action_ = 'raise_exception'
        run_batch(settings)
        assert len(self.cstream.contents) == 5, self.cstream.contents
        assert 'Traceback' in self.cstream.contents[2].strip()

    def test_quiet_flag(self):
        res = run_bch('info_console_output.conf', '-q', 'test')
        eq_(res.stdout.count('\n'), 0)

    def test_cmd_line_verbosity_takes_precedence_over_config_file(self):
        res = run_bch('info_console_output.conf', '-vvv', 'test')
        assert 'which maps to TestUseOnly' in res.stdout

    def test_cmd_line_vv_includes_stack_trace(self):
        res = run_bch('info_console_output.conf', '-v', 'error')
        assert 'Traceback' not in res.stdout

        res = run_bch('info_console_output.conf', '-vv', 'error')
        assert 'Traceback' in res.stdout

class TestFileLogging(object):

    def test_default_logging_is_not_setup(self):
        res = run_bch('default_file_logging.conf', 'test')
        assert len(res.files_created) == 0

    def test_minimal(self):
        res = run_bch('minimal_file_logging.conf', 'test')
        assert len(res.files_created) == 1
        lfile = res.files_created['blazech.log']
        assert 'batch started' in lfile
        assert 'no jobs have been configured' in lfile
        assert 'batch completed' in lfile
        as_list = open(lfile.full).read().strip().split('\n')
        assert len(as_list) == 3, as_list

    def test_error_file(self):
        res = run_bch('error_file_logging.conf', 'test')
        assert len(res.files_created) == 2
        lfile = res.files_created['blazech.log']
        as_list = open(lfile.full).read().strip().split('\n')
        assert len(as_list) == 4, as_list
        lfile = res.files_created['blazech_error.log']
        assert 'job test raised an exception' in lfile
        assert 'Traceback' in lfile



from nose.plugins.skip import SkipTest
from .helpers import run_cmdapp

def test_no_commands_given_is_error():
    result = run_cmdapp(expect_error=True)
    assert 'bch-test-app: error: too few arguments' in result.stderr, str(result)

def test_help():
    result = run_cmdapp('--help')
    assert 'usage: bch-test-app [-h] [-v] [-q] {cahw,cahwu,foo}' in result.stdout, str(result)

def test_invalid_command():
    result = run_cmdapp('notthere', expect_error=True)
    assert "argument subparser: invalid choice: 'notthere' (choose from 'cahw', 'cahwu', 'foo')" in result.stderr, str(result)

def test_verbosity_too_high():
    result = run_cmdapp('-vvvv', 'cahw', expect_error=True)
    assert 'verbosity too high, max is -vvv' in result.stderr, str(result)

def test_system_command():
    result = run_cmdapp('cahw')
    assert 'hello world from system command' in result.stdout, str(result)

def test_user_command():
    result = run_cmdapp('cahwu')
    assert 'hello world from user command' in result.stdout, str(result)

def test_plugin_command():
    result = run_cmdapp('foo', load_test_plugins = True)
    assert 'entry-point foo' in result.stdout, str(result)



from blazech.application import Application
from blazech.commands import BaseCommand
from ...tests.cmdapp import commands

def app_factory(initial_args=None):
    app = Application('cmdapp')
    #app.load_commands(commands)
    #app.load_commands(globals())
    app.run_cmd(initial_args)

class Baz(BaseCommand):
    name = 'baz'

    def execute(self, args):
        print 'local baz'




from nose.plugins.skip import SkipTest
from .helpers import run_bch

def test_no_commands_found_error():
    result = run_bch(use_test_config='none', expect_error=True)
    assert 'Error: no commands found or registered' in result.stderr, str(result)

def test_no_commands_given_is_error():
    result = run_bch(expect_error=True)
    assert 'bch: error: too few arguments' in result.stderr, str(result)

def test_help():
    result = run_bch('--help')
    assert 'usage: bch [-h] [-v] [-q] {hwu,hw}' in result.stdout, str(result)

def test_invalid_command():
    result = run_bch('notthere', expect_error=True)
    assert "argument subparser: invalid choice: 'notthere' (choose from 'hwu', 'hw')" in result.stderr, str(result)

def test_verbosity_too_high():
    result = run_bch('-vvvv', 'hw', expect_error=True)
    assert 'verbosity too high, max is -vvv' in result.stderr, str(result)

def test_system_command():
    result = run_bch('hw')
    assert 'hello world from system command' in result.stdout, str(result)

def test_user_command():
    result = run_bch('hwu')
    assert 'hello world from user command' in result.stdout, str(result)



import sys
import os
from os import path

user_dir = os.path.expanduser('~')
system_config_file = user_config_file = system_log_file = user_log_file = \
    system_commands_dir = user_commands_dir = None

def set_paths(testing_dir=None):
    global system_config_file, user_config_file, system_log_file, user_log_file, \
        user_dir, system_commands_dir, user_commands_dir
    if testing_dir:
        system_config_file = path.join(testing_dir, 'system', 'blazech.conf')
        system_log_file = path.join(testing_dir, 'system', 'blazech.log')
        system_commands_dir = path.join(testing_dir, 'system')
        user_config_file = path.join(testing_dir, 'user', 'blazech.conf')
        user_log_file = path.join(testing_dir, 'user', 'blazech.log')
        user_commands_dir = path.join(testing_dir, 'user')
    else:
        if sys.platform == 'win32':
            sroot = os.environ.get('SYSTEMROOT', None)
            if sroot is None:
                sroot = path.abspath('/')
            else:
                # SYSTEMROOT => c:\windows, so take the dirname of that to get
                # the drive
                sroot = path.dirname(sroot)
            system_log_file = path.join(sroot, 'logs', 'blazech.log')
            system_commands_dir = path.join(sroot, 'etc', 'blazech')
            user_dir = os.environ.get('APPDATA', user_dir)
        else:
            sroot = '/'
            system_log_file = path.join(sroot, 'var', 'log', 'blazech.log')
            system_commands_dir = path.join(sroot, 'usr', 'local', 'lib', 'blazech')
        system_config_file = path.join(sroot, 'etc', 'blazech/blazech.conf')
        user_log_file = path.join(user_dir, '.blazech', 'blazech.log')
        user_commands_dir = path.join(user_dir, '.blazech')
        user_config_file = path.join(user_dir, '.blazech', 'blazech.conf')
set_paths()



import os, sys
import blazech
from blazech.config import DefaultConfig
from blazech.locations import set_paths
from blazeutils.config import QuickSettings
from blazeutils.helpers import tolist

here = os.path.dirname(os.path.abspath(__file__))
base_path = os.path.join(here, 'test-scratch')

from scripttest import TestFileEnvironment

def get_tfenv(load_test_plugins, use_test_config):
    osenv = os.environ.copy()
    if not load_test_plugins:
        osenv['BCH_LOAD_TEST_PLUGINS'] = '0'
    else:
        osenv['BCH_LOAD_TEST_PLUGINS'] = '1'
    osenv['BCH_USE_TEST_CONFIG'] = use_test_config
    return TestFileEnvironment(os.path.join(here, 'test-output'), environ=osenv)

class TestConfig(DefaultConfig):
    def __init__(self):
        DefaultConfig.__init__(self)
        self.console.verbosity = 'none'

def run_bch(*args, **kw):
    load_test_plugins = kw.pop('load_test_plugins', False)
    use_test_config = kw.pop('use_test_config', 'both')
    env = get_tfenv(load_test_plugins, use_test_config)
    if args and args[0].endswith('.conf'):
        args=list(args)
        configpath = get_config_path(args.pop(0))
        args = ['bch', '-c', configpath] + args
    else:
        args = ('bch',) + args
    env.clear()
    result = env.run(*args, **kw)
    return result

def run_cmdapp(*args, **kw):
    load_test_plugins = kw.pop('load_test_plugins', False)
    use_test_config = kw.pop('use_test_config', 'both')
    env = get_tfenv(load_test_plugins, use_test_config)
    if args and args[0].endswith('.conf'):
        args=list(args)
        configpath = get_config_path(args.pop(0))
        args = ['bch-test-app', '-c', configpath] + args
    else:
        args = ('bch-test-app',) + args
    env.clear()
    result = env.run(*args, **kw)
    return result

def run_batch(config, *args):
    if not args:
        args = ('test', )
    if isinstance(config, QuickSettings):
        config.lock()
        return blazech.Batch(config, args).run()
    else:
        configpath = get_config_path(config)
        args = ['-c', configpath] + list(args)
        return blazech.run_batch(args)

# tell Nose not to test anything in here and test it
__test__ = False
def test_should_not_run():
    assert False

def get_config_path(fname):
    return os.path.join(here, 'configs', fname)

def set_test_fpath_root(name='noconfigs'):
    test_path = os.path.join(here, 'configs', name)
    set_paths(test_path)
set_test_fpath_root()



from blazech.application import _BCHApp, Application
from blazech.commands import BaseCommand
from nose.tools import eq_

# don't let nose test it, so rename it
from .helpers import set_test_fpath_root as stfp

from ..tests.cmdapp import commands as cmdapp_commands_mod

def setup_module():
    stfp('both')

def test_gather_commands():
    cmds = _BCHApp()._commands
    eq_(cmds['hw'].__module__, 'command_hw')
    eq_(cmds['hwu'].__module__, 'command_hwu')
    eq_(len(cmds), 2)

def test_cmdapp_gather_commands():
    app = Application('cmdapp')
    cmds = app._commands
    assert len(cmds) == 3, cmds
    eq_(cmds['cahw'].__module__, 'cmdapp_cmd_hw')
    eq_(cmds['cahwu'].__module__, 'cmdapp_cmd_hwu')
    eq_(cmds['foo'].__module__, 'blazech.tests.cmdapp.entry_points')

    app.load_commands(cmdapp_commands_mod)
    assert len(cmds) == 4
    eq_(cmds['bar'].__module__, 'blazech.tests.cmdapp.commands')

    class LocalCmd(BaseCommand):
        name = 'lc'

    app.load_commands(locals())
    assert len(cmds) == 5
    assert cmds['lc'] is LocalCmd



import argparse

class CountAction(argparse.Action):
    def __call__(self, parser, namespace, values, option_string=None):
        current_count = getattr(namespace, self.dest, None)
        if current_count is None:
            setattr(namespace, self.dest, 1)
        else:
            setattr(namespace, self.dest, current_count + 1)

parser = argparse.ArgumentParser()

parser.add_argument(
    '-v', '--verbose',
    dest='verbosity',
    action=CountAction,
    nargs=0,
    help='set level of console verbosity (max = -vvv)')
parser.add_argument(
    '-q', '--quiet',
    dest='quiet',
    action='store_true',
    help='turn off console output')

subparsers = parser.add_subparsers(dest='subparser')
#parser.add_option(
#    '-c', '--config-file',
#    dest='config_files',
#    action='append',
#    help='Full or relative path to config file (can be specified more than once)')
#parser.add_option(
#    '-n', '--no-system-configs',
#    dest='use_system_configs',
#    action='store_false',
#    help='when set, system and user config files will not be loaded even if available')
#parser.add_option(
#    '--log',
#    dest='log',
#    metavar='FILENAME',
#    help='Log file where a complete (maximum verbosity) record will be kept')
#parser.add_option(
#    # Writes the log levels explicitely to the log'
#    '--log-explicit-levels',
#    dest='log_explicit_levels',
#    action='store_true',
#    default=False,
#    help=optparse.SUPPRESS_HELP)
#parser.add_option(
#    # The default log file
#    '--local-log', '--log-file',
#    dest='log_file',
#    metavar='FILENAME',
#    default=default_log_file,
#    help=optparse.SUPPRESS_HELP)



from blazech.commands import BaseCommand

class ProjectStat(BaseCommand):
    name = 'bar'

    def execute(self, args):
        print 'entry-point bar'

# without a name attribute, this class won't be recognized
class _CustomBase(BaseCommand):
    def donothing(self):
        pass



from decimal import Decimal
from nose.tools import eq_
from nose.plugins.skip import SkipTest
from blazech.config import parse_config_files
from blazech.exceptions import ConfigError
from helpers import get_config_path, set_test_fpath_root, run_batch

#this code is from pysjobs, hasn't been converted as part of bch
raise SkipTest

def teardown_module():
    set_test_fpath_root()

def test_parse_config_files():
    config = parse_config_files(
        True,
        get_config_path('basic1.conf'),
    )
    eq_(config.batch.logfile.verbosity, 2)
    eq_(config.batch.console.verbosity, 1)

def test_parse_config_files_overwrite_correctly():
    config = parse_config_files(
        True,
        (
            get_config_path('basic1.conf'),
            get_config_path('basic2.conf'),
        )
    )
    # setting from second file overwrites first
    eq_(config.batch.logfile.verbosity, 3)
    # setting from first still comes through
    eq_(config.batch.console.verbosity, 1)
    # default settings gets overriden
    eq_(config.logging.mainlog.path, 'blazech.conf')

def test_parse_config_decimal_conversion():
    config = parse_config_files(
        True,
        get_config_path('basic1.conf'),
    )
    eq_(config.batch.decimal_test, Decimal('1.23456'))

def test_no_configs():
    try:
        parse_config_files(True, list())
        True,
        assert False, 'should have raised ConfigError'
    except ConfigError, e:
        assert 'At least one configuration file is expected' == str(e)

def test_system_default_config():
    set_test_fpath_root('systemonly')
    config = parse_config_files(True, list())
    assert config.batch.is_system == 1
    assert config.batch.from_where == 'system'

def test_user_default_config():
    set_test_fpath_root('useronly')
    config = parse_config_files(True, list())
    assert config.batch.from_where == 'user'

def test_system_and_user_default_config():
    set_test_fpath_root('both')
    config = parse_config_files(True, list())
    assert config.batch.is_system == 1
    assert config.batch.from_where == 'user'

def test_no_system_config_files():
    set_test_fpath_root('both')
    config = parse_config_files(
        False,
        get_config_path('basic1.conf'),
    )
    eq_(config.batch.logfile.verbosity, 2)
    eq_(config.batch.console.verbosity, 1)
    assert not hasattr(config.batch, 'from_where')

def test_section_includes():
    config = parse_config_files(
        False,
        get_config_path('basic1.conf'),
    )
    # section local
    eq_(config.second.s4, 4)
    # from first included section and no overwrite of/by section local
    eq_(config.second.s3, 3)
    # from second included section and no overwrite of/by section local
    eq_(config.second.s6, 6)
    # section local after include takes precedence
    eq_(config.second.s2, 'foobar')
    # section local before include gets overwritten
    eq_(config.second.s1, 1)
    # second section overwrites first
    eq_(config.second.s5, 2)



import sys
import logging
import logging.handlers
from blazeutils.log import clear_handlers_by_attr
from exceptions import ConfigError

MINIMAL = (logging.INFO+logging.WARN)/2
VDEBUG = logging.DEBUG - 1
HANDLER_TAG = '__from_blazech__'

SETTINGS_MAP = {
    'minimal': MINIMAL,
    'info': logging.INFO,
    'debug': logging.DEBUG,
    'vdebug': VDEBUG
}

VERBOSITY_FLAG_MAP = (
    'minimal',
    'info',
    'debug',
    'vdebug',
)

CONSOLE_STREAM = None

class Logger(logging.Logger):

    def minimal(self, msg, *args, **kwargs):
        return self.log(MINIMAL, msg, *args, **kwargs)

    def vdebug(self, msg, *args, **kwargs):
        return self.log(VDEBUG, msg, *args, **kwargs)

logging.setLoggerClass(Logger)
logging.addLevelName(MINIMAL, 'MINIMAL')
logging.addLevelName(VDEBUG, 'VDEBUG')

def setup_logging(settings):
    # clear any previously setup handlers
    clear_handlers()

    # have to set the root logger lower than WARN (the default) or our
    # many of our logs will never be seen
    logging.root.setLevel(VDEBUG)

    class InfoOnly(logging.Filter):
        def filter(self, record):
             return record.levelno <= MINIMAL

    format_str = "%(asctime)s - %(levelname)s - %(name)s - %(message)s"

    if settings.logging.errorlog.path:
        formatter = logging.Formatter(format_str)

        error_handler = logging.handlers.RotatingFileHandler(
              settings.logging.errorlog.path,
              maxBytes=settings.logging.rotation.max_bytes,
              backupCount=settings.logging.rotation.backup_count,
        )
        setattr(error_handler, HANDLER_TAG, True)
        error_handler.setLevel(logging.WARN)
        error_handler.setFormatter(formatter)
        logging.root.addHandler(error_handler)

    mlverb = settings.logging.mainlog.verbosity
    if mlverb != 'none':
        log_verbosity = SETTINGS_MAP.get(mlverb, None)
        if log_verbosity is None:
            raise ConfigError('unrecognized value for logging.mainlog.verbosity: %s' % mlverb)

    if settings.logging.mainlog.path:
        formatter = logging.Formatter(format_str)

        ml_handler = logging.handlers.RotatingFileHandler(
              settings.logging.mainlog.path,
              maxBytes=settings.logging.rotation.max_bytes,
              backupCount=settings.logging.rotation.backup_count,
        )
        setattr(ml_handler, HANDLER_TAG, True)
        ml_handler.setLevel(log_verbosity)
        ml_handler.setFormatter(formatter)
        ml_handler.addFilter(InfoOnly())
        logging.root.addHandler(ml_handler)

    converb = settings.console.verbosity
    if converb != 'none':
        console_verbosity = SETTINGS_MAP.get(converb, None)
        if console_verbosity is None:
            raise ConfigError('unrecognized value for console.verbosity: %s', mlverb)

        formatter = logging.Formatter("%(message)s")
        con_handler = logging.StreamHandler(CONSOLE_STREAM or sys.stdout)
        setattr(con_handler, HANDLER_TAG, True)
        con_handler.setLevel(console_verbosity)
        con_handler.setFormatter(formatter)
        if not settings.console.inc_errors:
            con_handler.addFilter(InfoOnly())
        logging.root.addHandler(con_handler)

    class NullHandler(logging.Handler):
        def emit(self, record):
            pass
    setattr(NullHandler, HANDLER_TAG, True)
    logging.root.addHandler(NullHandler())

def clear_handlers():
    clear_handlers_by_attr(HANDLER_TAG)



import os
import sys
from setuptools import setup, find_packages

cdir = os.path.abspath(os.path.dirname(__file__))
README = open(os.path.join(cdir, 'readme.rst')).read()
CHANGELOG = open(os.path.join(cdir, 'changelog.rst')).read()

from blazech import __VERSION__

if sys.platform == 'win32':
    kw = dict(entry_points=dict(console_scripts=['bch=blazech:main']))
else:
    kw = dict(scripts=['scripts/bch'])

required_packages = ['BlazeUtils']
try:
    import argparse
except ImportError:
    required_packages.append('argparse')

setup(name='BlazeCommandHelper',
      version=__VERSION__,
      description="A framework that facilitates shell and batch scripting in Python",
      long_description=README + '\n\n' + CHANGELOG,
      classifiers=[
        'Development Status :: 4 - Beta',
        #'Development Status :: 5 - Production/Stable',
        'Intended Audience :: Developers',
        'Intended Audience :: System Administrators',
        'License :: OSI Approved :: BSD License',
      ],
      author='Randy Syring',
      author_email='rsyring@gmail.com',
      url='http://bitbucket.org/rsyring/blazech/',
      license='BSD',
      packages=find_packages(),
      install_requires=required_packages,
      zip_safe=False,
      **kw)



import imp
import os
import sys

import pkg_resources

from blazech.baseparser import parser, subparsers
from blazech.commands import BaseCommand
from blazech.config import parse_config_files
import blazech.locations as locs
from blazech.locations import set_paths
from blazech.log import setup_logging, VERBOSITY_FLAG_MAP

class Application(object):
    """
        A class that represents a command line application
    """
    def __init__(self, name, cmd_entry_point=None, cmd_file_prefix=None,
                 use_test_config=None):
        self.name = name
        self.cmd_entry_point = cmd_entry_point or '%s.commands' % self.name
        self.cmd_file_prefix = cmd_file_prefix or '%s_cmd_' % self.name

        # some environment setup
        self.use_test_config = os.environ.get('BCH_USE_TEST_CONFIG', None)
        if self.use_test_config:
            from blazech.tests.helpers import set_test_fpath_root
            set_test_fpath_root(self.use_test_config)

        # initilize commands
        self._commands = {}
        self._gather_commands()

    def _gather_commands(self):
        # from installed python packages
        for p in pkg_resources.iter_entry_points(self.cmd_entry_point):
            cmd = p.load()
            self._commands[cmd.name] = cmd

        # from the system path
        self._load_path_commands(locs.system_commands_dir)

        # from the user path
        self._load_path_commands(locs.user_commands_dir)

    def _load_path_commands(self, dpath):
        if not os.path.isdir(dpath):
            return
        for fname in os.listdir(dpath):
            full_path = os.path.join(dpath, fname)
            basename, ext = os.path.splitext(fname)
            if os.path.isfile(full_path) and basename.startswith(self.cmd_file_prefix) and ext == '.py':
                mod = imp.load_source(basename, full_path)
                self._commands[mod.Command.name] = mod.Command

    def load_commands(self, load_from):
        """
            ``load_from`` can be a module or dictionary (usually from globals()
            or locals())
        """
        if not isinstance(load_from, dict):
            try:
                load_from = vars(load_from)
            except TypeError, e:
                if 'vars() argument' not in str(e):
                    raise
                raise TypeError('Expected "load_from" to be a dictionary or module')

        for obj in load_from.values():
            try:
                if issubclass(obj, BaseCommand) and obj is not BaseCommand and obj.name is not None:
                    self._commands[obj.name] = obj
            except TypeError, e:
                if 'issubclass() arg 1 must be a class' not in str(e):
                    raise

    def run_cmd(self, initial_args=None):
        # settings setup
        settings = parse_config_files(True, None)

        # where do the args come from?
        if initial_args is None:
            initial_args = sys.argv[1:]

        # add the sub-commands to the parser
        commands = self._commands.values()
        commands.sort(key=lambda x:x.name)
        for cmd in commands:
            cmd(subparsers)

        # if there are no subparsers, then we need to exit
        if len(subparsers._get_subactions()) == 0:
            parser.error('Error: no commands found or registered')

        # parse the arguments
        args = parser.parse_args()

        # verbosity and quite sanity checks
        if args.verbosity > 3:
            parser.error('verbosity too high, max is -vvv')
        if args.quiet and args.verbosity:
            parser.error('can not set quiet and verbose options')

        # verbosity log setup
        if args.quiet:
            settings.console.verbosity = 'none'
        if args.verbosity > 0:
            settings.console.verbosity = VERBOSITY_FLAG_MAP[args.verbosity]
            if args.verbosity >= 2:
                settings.console.inc_errors = True

        # log setup
        setup_logging(settings)

        # call the function the command told us to call
        args.func(parser, args)

class _BCHApp(Application):
    def __init__(self):
        Application.__init__(self, 'blazech', cmd_file_prefix='command_')



import logging
import os
from os.path import join as pjoin
import shlex
import subprocess
import zipfile
from blazeutils.helpers import csvtolist

class BaseAction(object):
    
    def __init__(self, batch, settings):
        self.batch = batch
        self.settings = settings
        
        self._setup_logging()
    
    def _setup_logging(self):
        cname = self.__class__.__name__
        self.log = logging.getLogger('blazech.actions.%s' % cname)
    
    def execute(self):
        pass

class TestUseOnly(BaseAction):
    """ only use this for testing"""

    def execute(self):
        self.log.info('test output')
        
class RaiseException(BaseAction):
    """ only use this for testing"""
    
    def execute(self):
        assert False, 'this is meant for testing'

class SystemCommand(BaseAction):
    
    def runcmd(self, args, stdin=''):
        proc = subprocess.Popen(
                args,
                stdin=subprocess.PIPE,
                stderr=subprocess.PIPE,
                stdout=subprocess.PIPE,
            )
        return proc.communicate(stdin)
    
    def execute(self):
        cmd_str = self.settings.cmd
        args = shlex.split(cmd_str)
        self.log.debug('executing: %s', cmd_str)
        self.log.vdebug('cmd string parsed as: %s' % args)
        stdout, stderr = self.runcmd(args)
        self.log.debug('cmd stdout: %s', stdout)
        self.log.debug('cmd stderr: %s', stderr)       
        
class DatabaseBackupBase(SystemCommand):
    
    @property
    def dbnames(self):
        try:
            touse = csvtolist(self.settings.include)
            self.log.debug('using dbnames from include setting: %s', touse)
        except AttributeError, e:
            if 'no attribute \'include\'' not in str(e):
                raise
        all_db_names = self.fetch_all_dbnames()
        self.log.debug('fetched db names: %s', all_db_names)
        try:
            for dbname in csvtolist(self.settings.exclude):
                all_db_names.remove(dbname)
            self.log.debug('excluded dbnames: %s', self.settings.exclude)
        except AttributeError, e:
            if 'no attribute \'exclude\'' not in str(e):
                raise
            
    def sql_file_path(self, dbname):
        return path.join(self.settings.target_dir, dbname + '.sql')
        
    def zip_file_path(self, dbname):
        return path.join(self.settings.target_dir, dbname + '.zip')

    def perform_compression(self, dbname):
        if self.file_compression == 'zip':
            zipf = zipfile.ZipFile(destinationfile, "w", zipfile.ZIP_DEFLATED )
            sql_file = self.sql_file_path()
            zipf.write(sqlfile, os.path.basename(sql_file) )
            zipf.close()
            if self.after_compression == 'delete_originals':
                os.unlink(sql_file)
    
    def execute(self):
        self.dbport = getattr(self.settings, 'port', 3306)
        self.file_compression = getattr(self.settings, 'file_compression', 'zip')
        self.after_compression = getattr(self.settings, 'after_compression', 'delete_originals')
        
        for dbname in self.dbnames:
            self.perform_backup(dbname)
            self.perform_compression(dbname)

class MysqlBackup(DatabaseBackupBase):
    
    def fetch_all_dbnames(self):
        try:
            import MySQLdb
            db = MySQLdb.connect(
                host=self.settings.host, 
                user=self.settings.user, 
                passwd=self.settings.password, 
                db="mysql", 
                port=self.dbport
            )
        except Exception, e:
            self.log.exception('mysql connection failed: %s', e)
            raise JobError

        cursor = db.cursor()
        cursor.execute("show databases")
        mysql_database_listings = cursor.fetchall()
        cursor.close()
        
        dbnames = []
        for row in mysql_database_listings:
            dbnames.append(row[0])
        
        return dbnames
    
    def cmdargs(self, dbname):
        return (
            pjoin(self.settings.bin_path, 'mysqldump'),
            '-h',
            self.settings.host,
            '-u',
            self.settings.user,
            '-p',
            self.settings.password,
            'r',
            self.sql_file_path(dbname),
            '-P',
            self.dbport,
            dbname
        )
    
    def perform_backup(self, dbname):
        self.runcmd(self.cmdargs(dbname))
